package wyklad.wyklad2.x02action;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

/**
 *
 * Pomysł jest taki: skoro w wielu aplikacjach mamy sytuacje że pewne akcje są wykonywane z kilku miejsc (na przykład:
 * ustawienie wyłuszczenia można ustawić z menu Format, za pomocą Ctrl+B, i za pomocą guzika na toolbarze), to możeby
 * definiować te akcje w jednym miejscu.
 *
 * Do tego służą klasy dziedziczące po Action. Z takiej akcji tworzy się potem guzik, menuItem itp. Akcja zawiera
 * informacje takie jak:
 *
 *   * nazwa
 *   * ikona
 *   * funkcja do wykonania
 *
 * I własności te są dla uzika czy menu item pobierane z akcji a nie ustawiane ręcznie. W programowani mówmimy o
 * pisaniu kodu DRY (Do Not Repeat yourself).
*/
public class ActionFrame extends JFrame{

   public ActionFrame() throws HeadlessException {

      setSize(200, 300);

      setDefaultCloseOperation(DISPOSE_ON_CLOSE);

      //Tworzymy dwie akcje. Te akcje będą wielokronnie wykorzystywane!
      final Action redAction = new MyAction("Ustaw na czerwone", Color.RED, this);

      final Action blueAction = new MyAction("Ustaw na niebieskie", Color.BLUE, this);

      setLayout(new FlowLayout());

      // Na podstawie akcji można stworzyć guzik
      add(new JButton(redAction));
      add(new JButton(blueAction));

      // Albo pozycję w menu!
      JMenuItem redItem = new JMenuItem(redAction);
      JMenuItem blueItem = new JMenuItem(blueAction);

      JCheckBoxMenuItem menuItem = new JCheckBoxMenuItem("Wyłącz akcje!");
      
      menuItem.addItemListener(new ItemListener() {
          @Override
          public void itemStateChanged(ItemEvent e) {
              boolean enabled = e.getStateChange() != ItemEvent.SELECTED;
              redAction.setEnabled(enabled);
              blueAction.setEnabled(enabled);
          }
      });

      redItem.setAccelerator(KeyStroke.getKeyStroke("control R"));

      JMenuBar menuBar =  new JMenuBar();

      setJMenuBar(menuBar);

      JMenu menu = new JMenu("Akcje");

      menuBar.add(menu);

      menu.add(redItem);
      menu.add(blueItem);
      menu.add(menuItem);

      pack();

   }

   public static void main(String[] args) {
      SwingUtilities.invokeLater(new Runnable() {
         public void run() {
            ActionFrame frame = new ActionFrame();
            frame.setVisible(true);
         }
      });
   }
  
}

class MyAction extends AbstractAction{

   final Color color;

   final JFrame parent;

   MyAction(String name, Color color, JFrame parent) {
      super(name);
      this.color = color;
      this.parent = parent;
   }

   public void actionPerformed(ActionEvent e) {
      parent.getContentPane().setBackground(color);
   }
}