package wyklad.wyklad2.x04slider;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.util.Hashtable;

/**
 * Przesuwane paski.
 *
 * Created by: Jacek Bzdak
 */
public class JSlisderFrame extends JFrame{
      
   
   private int[] colorParts = new int[3];
   
   public JSlisderFrame() {

      setSize(200, 200);

      colorParts[0] = getContentPane().getBackground().getRed();
      colorParts[1] = getContentPane().getBackground().getGreen();
      colorParts[2] = getContentPane().getBackground().getBlue();

      setLayout(new GridBagLayout());

      JSlider redSlider = new JSlider(0, 255);
      redSlider.setValue(colorParts[0]);

      redSlider.setMajorTickSpacing(16);
      redSlider.setMinorTickSpacing(4);

      redSlider.setPaintTicks(true);
      redSlider.setPaintLabels(true);

      JSlider greenSlider = new JSlider(0, 255);      
      greenSlider.setValue(colorParts[1]);

      JSlider blueSlider = new JSlider(0, 255);

      blueSlider.setValue(colorParts[2]);

      /**
       * Hash table używamy kiedy musimy --- w tym wypadku metoda setLabelTable 
       * nie przyjmuje instancji Map których normalnie powinno się używać jako 
       * kontenery. 
       */
      Hashtable labelTable = new Hashtable();
      
      labelTable.put( 0 , new JLabel("Brak niebieskiego"));
      labelTable.put( 45 , new JLabel("Mało niebieskiego"));
      labelTable.put( 250 , new JLabel("Mnóstwo niebieskiego"));
      
      blueSlider.setLabelTable( labelTable );
      blueSlider.setOrientation(JSlider.VERTICAL);
      blueSlider.setPaintLabels(true);

      redSlider.addChangeListener(new SliderListener(0, true));
      greenSlider.addChangeListener(new SliderListener(1, true));
      blueSlider.addChangeListener(new SliderListener(2, false));

      GridBagConstraints redConstraints = new GridBagConstraints();
      
      redConstraints.gridx = 0;
      redConstraints.gridy = 1;
      redConstraints.ipadx = 400;
      redConstraints.fill = GridBagConstraints.HORIZONTAL;

      add(redSlider, redConstraints);

      GridBagConstraints greenConstraints = new GridBagConstraints();

      greenConstraints.gridx = 0;
      greenConstraints.gridy = 3;
      greenConstraints.fill = GridBagConstraints.HORIZONTAL;

      add(greenSlider, greenConstraints);

      GridBagConstraints blueConstraints = new GridBagConstraints();

      blueConstraints.gridx = 1;

      blueConstraints.gridy = 1;
      blueConstraints.ipady = 100;

      blueConstraints.gridheight = 3;
      greenConstraints.fill = GridBagConstraints.VERTICAL;

      add(blueSlider, blueConstraints);
      
      
   }


   class SliderListener implements ChangeListener{

      /*
        Czy dany listener odpowiada za składową R, G czy B
      */
      final int part;

      /**
       * Czy akceptujemy eventy w których value is adjusitng
       */
      final boolean acceptAdjusting;

      SliderListener(int part, boolean acceptAdjusting) {
         this.part = part;
         this.acceptAdjusting = acceptAdjusting;
      }

      public void stateChanged(ChangeEvent e) {
         // Zanstanawialiście się Państwo dlaczego Bold Action z JEditorPane (w klasie cx.ath.jbzdak.wyklad1.editor.EditorFrame), 
         // Nie usiała znać referencji do JEditorPane? Pobrała sobie je za pomocą getSource()!
         JSlider slider = (JSlider) e.getSource();

         if(slider.getValueIsAdjusting() && ! acceptAdjusting ){
            return;
         }

         colorParts[part] = slider.getValue();

         Color color = new Color(colorParts[0], colorParts[1], colorParts[2]);
         getContentPane().setBackground(color);
      }
   }


   public static void main(String[] args) {
      SwingUtilities.invokeLater(new Runnable() {
         public void run() {
            JSlisderFrame frame = new JSlisderFrame();
            frame.setVisible(true);
         }
      });
   }

   
}


